/***************************************************************************
 *
 * R.Lacoste's PIC17C756 simple in-circuit programmer
 *
 * What is it ?
 *
 *   This software allow an in-circuit programming of the PIC17C756 processors
 *   through a specific hardware interface connected to the LPT port of any
 *   PC compatible computer (or so).
 *
 * Usage :
 *
 * 	 17c756pg b			Blank check
 * 	 17c756pg v hop.hex	Verify (compare chip to hex file)
 *	 17c756pg p hop.hex	Program chip with data from hex file
 *	 17c756pg a hop.hex	Auto mode (blank/program/verify)
 *	 17c756pg t			Test connection to the chip         
 *	 17c756pg z			Debug test (should not be usefull...)
 *
 *   The "t" mode check if the interface and the PIC link are ok. It's a good
 *   starting point, and in fact this test in done first for all other options.
 *
 *   The "z" mode is a repetitive access to the PIC, even if there is errors,
 *   allowing to use a scope and to debug the interface or the software. Press
 *   any key to skip to the next test. See source code for mode details.
 *
 * Warnings : 
 * 
 *   This is really a SIMPLE software ! Known limitations are :
 *
 *   - No read mode (as i didn't need it... but should be easy to add !)
 *   - Only one hex file format supported : HEX32
 *   - Configurations bits must be embedded in the hex file (address FE00)
 *   - Prototype grade programmer (as opposed to production quality programmer,
 *     ie not 100% compliant to Microchip's programming specifications... but
 *     seems working !).    
 *   - No support for 17C752 (but I think with only one line changed it should 
 *     work... See NBWORD definition. Not tested)
 *   - This programmer is an "in-circuit" programmer, and not a stand-alone one.
 *     No major differences however, except that the PIC is not supposed to
 *     be powered by the programmer. Just add a socket and Vcc should
 *     give a stand-alone programmer.
 *
 * The interface :
 *
 *   A specific hardware interface is required. See Postscript schematic
 *   for details (file 17c756pg.pst. Copy this file to a postscript printer
 *   or use the best and free postscript viewer : GNU's Ghostscript).
 *
 *   The crystal value (indicated as 5.0688MHz) is not critic. Any value from
 *   4 to 8MHz should work (but not tested)
 *
 *   This hardware interface should be connected to LPT1 (if you prefer LPT2,
 *   change it in the source). 
 *
 *   Given the small number of components, it should be possible to make this
 *   interface without a PCB. I'm used to wire-wrapping but use your favorite
 *   method ! Of course, double check your design before connecting it to a
 *   PIC, even if PIC's are quite cheap... The "t" option is very useful for
 *   this purpose.
 *                                                 
 * Connection to the PIC :
 *
 *   The interface is terminated by a 8 pin header. This header have to be
 *   connected to the PIC with the following assignments :
 *
 *  Pin 1 (VCC)       : to be connected to the PIC's power supply. 
 *                      Warning : this is not a power supply pin but a power 
 *                      supply sense pin, as the PIC is supposed to be powered
 *                      by its own power supply
 *  Pin 2 (RA1/TOCKI) : Programming clock. To be connected to the corresponding
 *                      pin of the PIC (pin 44 if PLCC)
 *  Pin 3 (RA4/RX)    : Data I/O. To be connected to the corresponding pin
 *                      of the PIC (pin 43 if PLCC)
 *  Pin 4 (RA5/TX)    : Data clock. To be connected to the corresponding pin
 *                      of the PIC (pin 42 if PLCC)
 *  Pin 5 (Mode/RST)  : Mode and reset control. To be connected to the 
 *                      corresponding pin of the PIC (pin 16 if PLCC). Be
 *                      careful : this pin should be pulled-UP when the
 *                      programmer is disconnected (6K8 between this pin and
 *					    Vcc on the application board)
 *  Pin 6 (TEST)      : test pin, allowing serial programming mode 
 *                      initialisation. To be connected to the 
 *                      corresponding pin of the PIC (pin 17 if PLCC). Be
 *                      careful : this pin should be pulled-DOWN when the
 *                      programmer is disconnected (6K8 between this pin and
 *					    GND on the application board)
 *  Pin 7             : not connected (useful as an index...)
 *  Pin 8 (GND)       : Ground. To be connected to PIC's Vss.
 *
 * How to use it ?
 *
 *   0/ Before any connection, the PIC application board should be SWITCHED OFF.
 *   1/ Connect the programming interface to the PC parallel port and to a 13V
 *      power supply (but not yet to the PIC), switch-on the 13V power supply
 *   2/ Launch the programming software with the required options :
 *        17C756PG A FOO.HEX
 *   3/ When required by the program, connect the header to the PIC and switch
 *      on the PIC power supply. Press any key.
 *   4/ Wait for the operation to complete
 *   5/ When required by the program, switch off the PIC power supply and
 *      disconnect the header. Press any key to leave the program
 *   6/ return to (2) if you have another operation to do or switch off the 13V
 *      power supply
 *
 *   (remarq : the header can be left in place between 2 operations)
 *
 * How it works ?
 *
 *   In order to start the PIC in in-circuit serial programming mode, it should
 *   be get up from reset with TEST and MODE pins at Vpp level (13V). When 
 *   launched, the programming software make sure that the MODE/RST pin is
 *   low (reset mode) before connecting the header and switch off the PIC. So
 *   the PIC is hold in Reset mode. Then the software put TEST to Vpp, and then
 *   MODE to Vpp. The PIC jump to serial programming mode and wait for its
 *   commands from the serial RA4/RA5 interface. Give a look to Microchip's
 *   programming specs for mode details (available at http://www.microchip.com)
 *
 *   Nota : in this serial programming mode, the PIC gets its clock from the 
 *   RA1 pin. So don't bother, it will works whatever your clock system is...
 *
 * More informations :
 *
 *   Compiler used : Visual C++ 1.52 (MS-DOS EXE mode)
 *                   Should be easy to use DJGPP or Borland but not tested.
 *
 *   Copyright (C) 1997 Robert Lacoste - France.
 *   Email : RLACOSTE@EuropeM01.nt.com
 *
 *   Nota : This is my work Email, but the writing of this software is not 
 *   related at all with my job.
 *
 *   Largely inspired (and copied !) from Paul Vollebregt'PGM74 software, of
 *   course with his aggreement. Thanks Paul !
 *
 *   Before any modification to this program, please first read carefully 
 *   Microchip's programming specifications (DS30274A)...
 *
 *   This program is free software.  Permission is granted to use,
 *   copy, or redistribute it as long as :
 *   - it is always distributed for free (except distribution costs)
 *   - full source code and documentation is distributed along with the 
 *     executable file, 
 *   - my copyright notice is included without any modification.
 *
 *   THIS PROGRAM IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND.
 *
 *   If you use this programmer, I will always be happy to receive
 *   your comments. Feels free to send me a small Email !
 *
 ****************************************************************************/
     

// Define NOSTOP to continue even if there is fatal errors during access to PIC
// usefull for debug...
//#define NOSTOP

     
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <string.h>
#include <time.h>                                         


// Line assignements on the parallel port

#define IN      32		/* line PAPEREND */

#define TESTVPP 1       /* line D0 */
#define MODEVPP 2       /* line D1 */
#define BUFCLK	4		/* line D2 */
#define OUT     32      /* line D5 */
#define WRENA	64      /* line D6 */
#define CLK     128     /* line D7 */


// Parallel port access functions

#define inbit           (_inp(s_reg)&IN)
#define testvppon       _outp(d_reg,(d_bits |= TESTVPP))
#define testvppoff      _outp(d_reg,(d_bits &= ~TESTVPP))
#define modevppon       _outp(d_reg,(d_bits |= MODEVPP))
#define modevppoff      _outp(d_reg,(d_bits &= ~MODEVPP))
#define bufclkhi        _outp(d_reg,(d_bits |= BUFCLK))
#define bufclklo        _outp(d_reg,(d_bits &= ~BUFCLK))
#define clkhi           _outp(d_reg,(d_bits |= CLK))
#define clklo           _outp(d_reg,(d_bits &= ~CLK))
#define outhi           _outp(d_reg,(d_bits |= OUT))
#define outlo           _outp(d_reg,(d_bits &= ~OUT))
#define wrenaon         _outp(d_reg,(d_bits |= WRENA))
#define wrenaoff        _outp(d_reg,(d_bits &= ~WRENA))


// Global variables

int d_bits = 0;		// Current output port value
int d_reg;			// data register address
int s_reg;			// Status register address
                                        
                                        
//Number of program memory words : 16x1024 for 17C756, 8x1024 for 17X752 
#define NBWORDS (16*1024)			// 16Kwords

unsigned short romimage[NBWORDS];	// Rom image
unsigned short confimage;			// configuration word

int check=0;						// Checksum of loaded data


//---------------------------------------------------------------------------------------
// Millisecond routines. Thanks Paul !
//---------------------------------------------------------------------------------------
       
#define asm __asm                 // For visual C++

/* 5sec */
void microdelaytest(void)
{
   asm     pushf
   asm     push ax
   asm     push cx
   asm     mov  cx,5              /* 5 * 65535 * 15.08 us = 5 sec.*/

next3:
   asm	   push cx
   asm     mov cx,0xFFFF          /* 65535 * 15.08 us = 1 sec.      */
   asm     in  al,0x61
   asm     and al,0x10
   asm     mov ah,al

next2:
   asm     push cx                /* Timeout 100uS if no puls is detected.*/
   asm     mov cx,80              /* 80 * ca 1.25uS = 100 uS.*/
next1:                            /* test 0x61 for 100 uS, if still*/
   asm     in  al,0x61		  	  /* equal then error and exit.*/
   asm     and al,0x10
   asm     cmp ah,al
   asm     loope next1
   asm     jz error               /*  Q, timeout ?.*/

   asm     pop cx
   asm     mov ah,al
   asm     loop next2

   asm     pop  cx
   asm     loop next3

   asm     jmp exit1

error:
   asm     pop cx                 /* Timeout, clear stack. */
   asm     pop cx

exit1:
   asm     pop cx
   asm 	   pop ax
   asm     popf
}


/* True PC speed independent 100uS delay.         */
/* Timeout is supported for noncompliant hardware.*/
void microdelay(void)
{
   asm     pushf
   asm     push ax
   asm     push cx
   asm     cli
   asm     mov cx,3       /* 3 * 15 us = 45.0 us + command(ENDPRG) = 100us*/
   asm     in  al,0x61
   asm     and al,0x10
   asm     mov ah,al

next2:
   asm     push cx                /* Timeout 100uS if no puls is detected. */
   asm     mov cx,80              /* 80 * ca 1.25uS = 100 uS.              */
next1:                            /* test 0x61 for 100 uS, if still        */
   asm     in  al,0x61		  /* equal then error and exit.            */
   asm     and al,0x10
   asm     cmp ah,al
   asm     loope next1
   asm     jz error

   asm     pop cx
   asm     mov ah,al
   asm     loop next2
   asm     jmp exit2

error:
   asm     pop cx                 /* Timeout, clear stack.*/

exit2:
   asm     pop cx
   asm     pop ax
   asm     popf
}

                                                                                         
                                                                                         
//--------------------------------------------------------------------------------------
// Parallel port setup
//--------------------------------------------------------------------------------------
                                                                                         
void lptsetup(void)
{
   d_reg = *((short __far *)0x408);  /* find the base address of LPT1 port */
   s_reg = d_reg+1;

   switch ( d_reg ) {  /* I don't want to blow anything up, so check port */
       case 0x3BC:
       case 0x378:
       case 0x278: break;
       default:  printf("LPT at 0x%04X\n",d_reg);
		 printf("Unlikely LPT address !\n");
		 exit (1);
   }

   printf("LPT port found at 0x%04X\n",d_reg);
}


//--------------------------------------------------------------------------------------
// Elementary synchronous serial port access routines
//--------------------------------------------------------------------------------------
   
void init_port ()
{                   
   clklo;                                          
   bufclklo;
   bufclkhi;
   bufclklo;
   outlo;
   testvppoff;
   modevppoff;
   wrenaoff;
   printf("Output port initialized.\n\n");
   printf("  1/ Connect header to PIC,\n");
   printf("  2/ Switch on PIC power supply, and \n");
   printf("  3/ Press ENTER (ctrl-C to leave) :");
   (void)getchar();  
   printf("\n");
}

void clock_out(char bit)
{
   if (bit)
     outhi; 
   else
     outlo;
   clkhi;          		// Setup time >15nS (P4)
   bufclkhi;
   bufclklo;
   clklo;   
   bufclkhi;
   bufclklo;
   outlo;               // Hold time >15nS (P5)
}

unsigned char clock_in(void)
{
   int b;   
   char foo;

   clkhi;               // Setup time >15nS (P4)
   bufclkhi;
   bufclklo;  
   foo=inp(s_reg);
   b = inbit? 1: 0;     // Hold time >15nS (P5)
   clklo;   
   bufclkhi;
   bufclklo;
   return b;
}

void out_byte(unsigned char w)
{
   int b;
   char foo;
   
   for ( b=0; b<8; ++b ) 
   {
     clock_out((char)(w&((char)1<<b)));
     foo=inp(s_reg);		// serial clock period > 0.85S (P3)
   }
}

unsigned char in_byte(void)
{
   int b;
   unsigned char w;
   char foo;

   for ( w=0, b=0; b<8; ++b )
   {
     w += clock_in()<<b;                                                   
     foo=inp(s_reg); 		// serial clock period > 0.85S (P3)
   }
   return w;
}

                
//--------------------------------------------------------------------------------------
// PIC17C756 access functions
//--------------------------------------------------------------------------------------

// Initialize in-circuit serial programming mode
void P17C_openICSP()
{               
   testvppon;   
   microdelay();	// wait more than 15S (P1)
   modevppon;                     
   microdelay(); 	// wait for mode than 20*0.85S (P2)
   microdelay();
   printf("PIC is now in ICSP mode\n");
}                                              

// End in-circuit serial programming mode
void P17C_closeICSP()
{               
   modevppoff;
   microdelay();
   testvppoff;  
   clklo;   
   bufclkhi;
   bufclklo;
   outlo;
   wrenaoff;
   printf("\nPIC is now stopped, all lines back in inactive state\n");
   printf("1/ Switch off PIC power supply\n");
   printf("2/ Disconnect header and\n");
   printf("3/ Press ENTER :");
   (void) getchar();
}
 
// Reset program memory pointer 
void P17C_resetaddress()
{          
   wrenaon;                                                                
   out_byte((unsigned char)0x00);
   microdelay();			// Wait for execution time (more than 165S) (P6)
}
                               
// Increment address
void P17C_incraddress()
{          
   wrenaon;
   out_byte((unsigned char)0x06);
   microdelay();			// Wait for execution time (more than 165S) (P6)
}

// Load address pointer
void P17C_loadaddress(unsigned short address)
{          
   wrenaon;
   out_byte((unsigned char)0x0A);
   microdelay();			// Wait for interbyte delay (more than 165S) (P7)
   out_byte((unsigned char)(address&0x00FF));
   out_byte((unsigned char)(address>>8));
   microdelay();			// Wait for execution time (more than 165S) (P6)
}

// Read address pointer
unsigned short P17C_readaddress()
{          
   unsigned char vlow;  
   char foo;
   unsigned short value;
   
   wrenaon;
   out_byte((unsigned char)0x0C);
   foo=inp(s_reg); 					// Wait for byte to non-driven delay (>0.85S) (P9)
   wrenaoff;                                                                         
   microdelay();					// Wait for command to read delay (>245S) (P8)
   vlow=in_byte();
   value= (((unsigned short)in_byte()<<8)+(unsigned short)vlow);
   microdelay();					// Wait for execution time (more than 165S) (P6)
   return value;
}

// Load data register
void P17C_loaddata(unsigned short data)
{
   wrenaon;
   out_byte((unsigned char)0x02);
   microdelay();			// Wait for interbyte delay (more than 165S) (P7)
   out_byte((unsigned char)(data&0x00FF));
   out_byte((unsigned char)(data>>8));
   microdelay();			// Wait for execution time (more than 165S) (P6)
}                                    

// Read data register
unsigned short P17C_readdata()
{
   unsigned char vlow;  
   char foo;
   unsigned short value;
   
   wrenaon;
   out_byte((unsigned char)0x04);
   foo=inp(s_reg); 					// Wait for byte to non-driven delay (>0.85S) (P9)
   wrenaoff;                                                                         
   microdelay();					// Wait for command to read delay (>245S) (P8)
   vlow=in_byte();
   value= (((unsigned short)in_byte()<<8)+(unsigned short)vlow);
   microdelay();					// Wait for execution time (more than 165S) (P6)
   return value;
}

// Begin programming
// Warning : any command must be sent to the PIC after this one to wake up it
void P17C_beginprogramming()
{
   wrenaon;
   out_byte((unsigned char)0x08);
   microdelay();					// Wait for execution time (1005S) (P10)
}


                                     
//--------------------------------------------------------------------------------------
// Hex32 file loading routine
//--------------------------------------------------------------------------------------                
                 
void reset_buffer ()
{
   int i;
   for (i=0;i<NBWORDS;i++)
     romimage[i]=0xFFFF;
   confimage=0xFFFF;
}
                 
int hexdigit(FILE *fp)
{
   int c;

   if ( (c=getc(fp)) == EOF )
   {
     printf("ERROR : Unexpected EOF in object file.",fp);
     exit(1);
   }

   c -= c>'9'? 'A'-10: '0';
   if ( c<0 || c>0xF )
   {
     printf("ERROR : Hex digit expected, error in object file.",fp);
     exit (1);
   }
   return c;
}

int hexbyte(FILE *fp)
{
   int b;

   b = hexdigit(fp);
   b = (b<<4) + hexdigit(fp);
   check += b;               /* nasty side-effect, but useful trick */
   return b;
}

unsigned hexword(FILE *fp)
{
   unsigned w;

   w = hexbyte(fp);
   w = (w<<8) + hexbyte(fp);
   return w;
}

#define swab(w) (((w)>>8)+(((w)&0xFF)<<8))   /* swap bytes */

void load_hex32file(char *filename)
{
   FILE *fp;
   int type, nwords, i;
   unsigned short address, w;           
   unsigned short addressoffset;      
   int flagconfbits=0;
                                                         
   printf("Loading file <%s>...\n",filename);
                                                            
   fp=fopen(filename,"r");					// Open file
   if (fp==NULL)
   {
     perror("ERROR : Unable to open input file");
     exit(1);
   }
                                   
   type = 0;                    
   addressoffset=0;
   while ( type != 1 )                    	// Until end of file marker (type 1)
   {
      if ( getc(fp) != ':' )                // Verify header
      {
        printf("ERROR : Expected ':', in object file. Not an HEX32 file ?\n",fp);
        fclose(fp);
        exit(1);
      }                                                     
      
      check = 0;
      w = hexbyte(fp);
      nwords = w/2;							// Number of words on the line
      
      w = hexword(fp);
      address = w/2+addressoffset;			// Address pointer
      
      type = hexbyte(fp);                   // Read type
      
      if (type==0)							// Data record
      {
        for ( i=0; i<nwords; ++i, ++address )
        {
         if ( address < NBWORDS )
         {
	       w = hexword(fp);
	       romimage[address] = swab(w);
         }
         else
         {
     	   if ( address != 0xFE00 )
	 	  {
	        printf("ERROR : Impossible address found in object file : %4X\n",address);
	        fclose(fp);
	        exit (1);
	      }
	      w = hexword(fp);
		  confimage = swab(w);
		  flagconfbits=1;
         }              
        }
       }
       
       else if (type==1)					// End of file record
       {}
       
       else if (type==2)					// Segment address record
       {
         printf("ERROR : Segment address record found in file. Not supported, sorry!\n");
         fclose(fp);
         exit(1);
       }       
         
       else if (type==4)					// Linear address record
       {                                                            
          if (nwords!=1)
          {
            printf("ERROR : Illegal linear segment record (more than 1 value)\n");
            fclose(fp);
            exit(1);
          }       
          
          w=hexword(fp);
          if((w!=0) && (w!=1))
          {
            printf("ERROR : Illegal linear segment record (value %d)\n",w);
            fclose(fp);
            exit(1);
          }                                                                
          
          addressoffset=32768u*w;
       }
       
       else
       {
         printf("ERROR : Unkown record type found in file (%d)\n",type);
         fclose(fp);
         exit(1);
       }
       
            
      (void) hexbyte(fp);
      (void) getc(fp);
      if ( check&0xFF )
      {
        printf("ERROR : Checksum error in object file.",fp);
        fclose(fp);
        exit(1);
      }
           
   }
   
   fclose(fp);          
   
   if(flagconfbits==0)
   {
     printf("ERROR : Configuration bits undefined in hex file\n");
     exit(1);
   }
}                                                                                         


void display_confbits()
{ 
  int v;
    
  printf("Conf.bits = ");
  
  // Decode clock source
  v=confimage&0x0003;   
  switch (v)
  {
    case 0 : printf ("Osc LF, "); break;
    case 1 : printf ("Osc RC, "); break;
    case 2 : printf ("Osc XT, "); break;
    case 3 : printf ("Osc EC, "); break;
  }
                                 
  // Decode watchdog mode
  v=(confimage>>2)&0x0003;   
  switch (v)
  {
    case 0 : printf ("Wdog dis, "); break;
    case 1 : printf ("Wdog ena/64, "); break;
    case 2 : printf ("Wdog ena/256, ");break;
    case 3 : printf ("Wdog ena/0, ");  break;
  }                           
  
  // Decode processor mode
  v=(confimage>>4)&0x0805;
  switch (v)
  {
    case 0x0000 : printf ("Microcontroler protected mode, "); break;
    case 0x0801 : printf ("Extended microcontroler mode, ");  break;
    case 0x0804 : printf ("Microcontroler mode, ");           break;
    case 0x0805 : printf ("Microprocessor mode, ");           break;
    default     : printf ("\nERROR : Illegal mode !\n");            
                  exit(1);
  }                           
                                 
  // Decode Boden mode
  v=(confimage&0x4000);                                 
  switch (v)
  {
    case 0x0000 : printf ("Boden dis\n"); break;
    case 0x4000 : printf ("Boden ena\n");  break;
  }                           
}


//--------------------------------------------------------------------------------------
// Computer timer verification and calibration
//--------------------------------------------------------------------------------------


short fulltestmode(void)
{
   time_t t;

   printf("\nThis is a computer PIT hardware test, this test takes a few sec.\n");
   t=time(NULL);
   microdelaytest();        /* test microdelay hardware using warp drive.*/
   t=time(NULL)-t;
   if ( t > 6 || t < 4)
   {
    switch(t)
    {
	case 0:
		printf("Sorry your computers PIT hardware isn't compliant with the IBM standard.\n");
		printf("Using the programmer is for your own risk and could damage the PIC !.\n");
		break;
	default:
		if (t <= 50)
		{
		 printf("Sorry your computers PIT hardware isn't quiet compliant with the IBM standard.\n");
		 printf("The PIC will be programmed with a puls of %duS instead of 100uS, this\n",(t*100)/5);
		 printf("puls is still within the limits specified by MICROCHIP.\n");
		}
		else
		{
		 printf("Sorry your computers PIT hardware isn't compliant with the IBM standard. Don't\n");
		 printf("using this computer to programmer the PIC, it will damage the PIC !.\n");
		}
		break;
     }
    return 0x01;
   }
   else
   {
    printf("Your computers PIT hardware is compliant with the IBM standard.\n");
    return 0x00;
   }
}

void MakeCompl(void)
{
   time_t t;

   printf("\nThis function will try to make your hardware IBM compliant it's possible\n");
   printf("that your computer will stop responding. If this happens then it's not\n");
   printf("possible to make your hardware IBM compliant !.\n");
   printf("Trying to make the hardware IBM compliant, this takes a few sec.\n");
   _disable();              /* modify timer 1.*/
   _outp(0x43,0x76);    /* set stuurwoord.*/
   _outp(0x41,18);      /* set LSB tellerwaarde. (15us) */
   _outp(0x41,0);       /* set MSB tellerwaarde.        */
   _enable();
   t=time(NULL);
   microdelaytest();        /* test microdelay hardware using warp drive.*/
   t=time(NULL)-t;
   if ( t > 6 || t < 4)                                                                 
   {
     printf("ERROR : Sorry it's impossible to make your computers PIT hardware compliant\n");
     exit(1);
   }
   else
     printf("Your computers PIT hardware is now compliant with the IBM standard.\n");
}
                                                                                         
                                                                                         
//---------------------------------------------------------------------------------------                         
// Functional blocks
//---------------------------------------------------------------------------------------
                                    
                                    
// "Debug tests" (usefull to bebug the programming interface...)
void debug_test()
{
  unsigned short v1,v2,v;

  printf("\nDebugging tests...\n");
  
  while(!_kbhit()) 
  {   
    P17C_loadaddress(0x1234);
    v=P17C_readaddress();
    printf ("Load address 0x1234, read 0x%04X\n",v);
  }
  (void)getchar();
  
  while(!_kbhit()) 
  {   
    P17C_resetaddress();
    v=P17C_readaddress();
    printf ("Reset address, read 0x%04X\n",v);
  }
  (void)getchar();
  
  while(!_kbhit()) 
  {   
    P17C_loadaddress(0x52C3);
    v=P17C_readaddress();
    printf ("Load address 0x52C3, read 0x%04X\n",v);
  }
  (void)getchar();

  while(!_kbhit()) 
  {   
    P17C_loadaddress(0xD41F);
    v=P17C_readaddress();
    printf ("Load address 0xD41F, read 0x%04X\n",v);
  }
  (void)getchar();
  
  while(!_kbhit()) 
  {   
    P17C_loadaddress(0x0000);
    v=P17C_readaddress();
    printf ("Load address 0x0000, read 0x%04X\n",v);
  }
  (void)getchar();
  
  while(!_kbhit()) 
  {   
    P17C_incraddress();
    v=P17C_readaddress();
    printf ("Increment address, read 0x%04X\n",v);
  }
  (void)getchar();

  while(!_kbhit()) 
  {   
    P17C_loadaddress(0xF000);
    v=P17C_readaddress();
    printf ("Load address 0xF000, read 0x%04X\n",v);
  }
  (void)getchar();
  
  while(!_kbhit()) 
  {   
    P17C_incraddress();
    v=P17C_readaddress();
    printf ("Increment address, read 0x%04X\n",v);
  }
  (void)getchar();

  
  P17C_resetaddress();
  while (!kbhit())
  {
    v1=P17C_readdata(); v2=P17C_readaddress();
    printf("read address 0x%04X, value 0x%04X\n",v2,v1);
    P17C_incraddress();
  }
  (void)getchar();  
}
                                                                                         
                                                                                         
// Test connection to the pic
void connection_test()
{                                                                                  
  unsigned short v;

  printf("\nConnection tests...\n");
                                                  
  // Load address test
  P17C_loadaddress(0x1234);
  if ((v=P17C_readaddress())!=0x1234)
  {
    printf ("ERROR : Load address test failed (found 0x%04X, expected 0x1234).\n",v);
    printf ("Verify connection to PIC\n");
#ifndef NOSTOP
    P17C_closeICSP(); 
    exit(1);  
#endif
  }         
  
  // Increment address test
  P17C_incraddress();
  if ((v=P17C_readaddress())!=0x1235)
  {
    printf ("ERROR : Increment address test failed (found 0x%04X, expected 0x1235).\n",v);
    printf ("Verify connection to PIC\n");
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }               
  
  // Reset address test
  P17C_resetaddress();
  if ((v=P17C_readaddress())!=0x0000)
  {
    printf ("ERROR : Reset address test failed (found 0x%04X, expected 0x0000).\n",v);
    printf ("Verify connection to PIC\n");
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }               

  printf("Connection test successfull !\n");
}


// Blank check the program memory
void blankcheck_program()
{                
  unsigned short i;             
  unsigned short v;
  
  printf("\nBlank-checking program memory... (*=1Ko) : ");
  
  P17C_resetaddress();
  for (i=0;i<NBWORDS;i++)
  {
    if ((v=P17C_readdata())!=0xFFFF)
    {
      printf("ERROR : Blank check failed at address 0x%04X, value 0x%04X\n",i,v);
#ifndef NOSTOP
      P17C_closeICSP();
      exit(1);  
#endif
    }         
    
    P17C_incraddress();
    
    if (i%1024==1023)
      printf("* ");
  }                    
  
  if ((v=P17C_readaddress())!=i)
  {
    printf("\nERROR : Bad end address pointer, found 0x%04X, expected 0x%04X\n",v,i);
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }                    
  
  printf("\nBlank check successfull !\n");
}
         
         
// Blank check the configuration bits
void blankcheck_confbits()
{     
  unsigned short v;
                      
  printf("\nBlank-checking configuration bits...\n");
  
  P17C_loadaddress(0xFE00);
  if ((v=P17C_readdata()) != 0xFFFF)
  {
    printf("ERROR : Bad value at 0xFE00, found 0x%04X\n",v);
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }                                                       
  
  P17C_loadaddress(0xFE08);
  if ((v=P17C_readdata()) != 0xFFFF)
  {
    printf("ERROR : Bad value at 0xFE08, found 0x%04X\n",v);
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }                                                       

  printf("Blank check successfull !\n");
}
         
         
// Program the program memory
void program_program()
{                                                  
  unsigned short i,j;             
  unsigned short v;
  unsigned short n;
  int progok;                               
  unsigned short maxN;
  
  printf("\nProgramming program memory... (1*=1Ko) : ");
           
  maxN=0;
  P17C_resetaddress();
  for (i=0;i<NBWORDS;i++)
  {
    if (romimage[i]!=0xFFFF)					// Don't program not programmed cells...
    {                                                                           
      n=1;
      
      P17C_loaddata(romimage[i]);				// Load data to be programmed
      
      progok=0;
      while (progok==0)							// Loop until programmed ok
      {                                                
        P17C_beginprogramming();				// Program
        if ((v=P17C_readdata())==romimage[i])   // and verify
        {
          progok=1;
        }         
        else
        {
          n=n+1;
          if (n>25)
          {
            printf("\nERROR : Impossible to program location 0x%04X, found 0x%04X, expected 0x%04X\n",
                   i,v,romimage[i]);
            P17C_incraddress();
#ifndef NOSTOP
            P17C_closeICSP();
    	    exit(1);  
#endif
			progok=1;
          }
        }
        
        if(n>maxN)          					// Store maximum number of retries
          maxN=n;                                                                 
      }

      for (j=0;j<3*n;j++)						// 3N overprogramming
        P17C_beginprogramming();
    }
    
   
    P17C_incraddress();							// Next address
    
    if (i%1024==1023)
      printf("* ");
  }                    
  
  if ((v=P17C_readaddress())!=i)
  {
    printf("\nERROR : Bad end address pointer, found 0x%04X, expected 0x%04X\n",v,i);
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }              
  
  printf("\nProgrammation successfull, maximum number of retries : %d\n",maxN);
}
         
         
// Program the configuration bits
void program_confbits()
{     
  int i,j;
  unsigned short v;
  
  printf("\nProgramming configuration bits...\n");
  
  P17C_resetaddress();
  P17C_loadaddress(0xFE00);
  
  for(i=0;i<16;i++)								// for the 16 bits, starting with lsb
  {                                                         
    if ((i<5) || (i==6) || (i==14) || (i==15))  // Don't program unusued bits
    {
      if ((confimage&(1<<i))==0)				// If bit is 0 (must be programmed)
       for(j=0;j<20;j++)						// Arbitrary, do 20 write cycles
       {
         P17C_beginprogramming();
       }
    }               
    
    P17C_incraddress();
  }

  if ((v=P17C_readaddress())!=0xFE00+i)
  {
    printf("ERROR : Bad end address pointer, found 0x%04X, expected 0x%04X\n",
           v,0xFE00+i);
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }              

  printf("Programmation successfull !\n");
}
         
         
// Verify the program memory
void verify_program()
{
  unsigned short i;             
  unsigned short v;

  printf("\nVerifying program memory... (1*=1Ko) : ");
  
  P17C_resetaddress();
  for (i=0;i<NBWORDS;i++)
  {
    if ((v=P17C_readdata())!=romimage[i])
    {
      printf("\nERROR : Verify failed at address 0x%04X, found 0x%04X, expected 0x%04X\n",
             i,v,romimage[i]);
#ifndef NOSTOP
      P17C_closeICSP();
      exit(1);  
#endif
    }         
    
    P17C_incraddress();
    
    if (i%1024==1023)
      printf("* ");
  }                    
  
  if ((v=P17C_readaddress())!=i)
  {
    printf("\nERROR : Bad end address pointer, found 0x%04X, expected 0x%04X\n",v,i);
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }                                       
  
  printf ("\nVerification successfull !\n");
}
         
         
// Verify the configuration bits
void verify_confbits()
{
  unsigned short v;
  
  printf("\nVerifying configuration bits...\n");

  P17C_loadaddress(0xFE00);
  if ((v=P17C_readdata()) != (confimage|0xFF00))
  {
    printf("ERROR : Bad value at 0xFE00, found 0x%04X, expected 0x%04X\n",
           v,(confimage|0xFF00));
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }                                                       
  
  P17C_loadaddress(0xFE08);
  if ((v=P17C_readdata()) != ((confimage>>8)|0xFF00))
  {
    printf("ERROR : Bad value at 0xFE08, found 0x%04X, expected 0x%04X\n",
           v, ((confimage>>8)|0xFF00));
#ifndef NOSTOP
    P17C_closeICSP();
    exit(1);  
#endif
  }                                                       
  
  printf ("Verification successfull !\n");
}
         
         
                                                                                         
//---------------------------------------------------------------------------------------                         
// Main program                                                                          
//---------------------------------------------------------------------------------------
                         
void main(int argc, char *argv[])
{     
  // Hello
  printf("PIC17C756 SIMPLE PROGRAMMER V1.1 - Robert Lacoste (redevant to Paul Vollebregt)\n\n");
                              
                              
  // Parameters verification
  if ((argc<2) 
   || (argc>3)
   || ((argc==2) && ((strcmp(argv[1],"b")!=0) && (strcmp(argv[1],"t")!=0)
                 &&  (strcmp(argv[1],"z")!=0)))
   || ((argc==3) && ((strcmp(argv[1],"v")!=0) && (strcmp(argv[1],"p")!=0) 
                                              && (strcmp(argv[1],"a")!=0))))
  {
    printf("\nUsage :\n\n");
    printf("    17c756pg b            Blank check\n");
    printf("    17c756pg v hop.hex    Verify (compare chip to hex file)\n");
    printf("    17c756pg p hop.hex    Program chip with data from hex file\n");
    printf("    17c756pg a hop.hex    Auto mode (blank/program/verify)\n");
	printf("    17c756pg t            Test connection to the chip\n");
	printf("    17c756pg z            Debug test (should not be usefull...)\n\n");
	printf("'hop.hex' must be an HEX32 formatted file.\n");
	printf("Configuration bits must be defined inside this file\n\n");
	exit(1);
  }
                        
  // File load
  reset_buffer();
  if (argc==3)
  {
    load_hex32file(argv[2]);
    display_confbits();
  }
                                
  // Computer test and timer definition
  if (fulltestmode())
    MakeCompl();
                                       
  // LPT port assignement
  printf("\n");
  lptsetup(); 
  
  // Initialisation
  init_port();  
  P17C_openICSP();
  
  // Selection of the required function
  if (strcmp(argv[1],"b")==0)
  {
    connection_test();
    blankcheck_program();
    blankcheck_confbits();
  }
  else if (strcmp(argv[1],"v")==0)
  {
    connection_test();
    verify_program();
    verify_confbits();
  }
  else if (strcmp(argv[1],"p")==0)
  {
    connection_test();
    program_program();
    program_confbits();
  }
  else if (strcmp(argv[1],"a")==0)
  {
    connection_test();
    blankcheck_program();
    blankcheck_confbits();
    program_program();
    program_confbits();
    verify_program();
    verify_confbits();
  
  }
  else if (strcmp(argv[1],"t")==0)
  {                       
    connection_test();
  }
  else if (strcmp(argv[1],"z")==0)
  {                       
    debug_test();
  };
  
  P17C_closeICSP();
  printf("\nGoodbye !\n\n");
}
                                                                                         

//----------------------------------- END OF FILE ----------------------------------------                                                                                         
